#include <iostream>
#include <cstring>
#include "fitsio.h"
using namespace std;

void get_naxis(char *in_filename, int naxis[])
{
  int status = 0;
  long naxis1, naxis2;
  fitsfile *fptr;

  fits_open_file(&fptr, in_filename, READONLY, &status);

  fits_read_key_lng(fptr, "NAXIS1", &naxis1, 0, &status);
  fits_read_key_lng(fptr, "NAXIS2", &naxis2, 0, &status);

  fits_close_file(fptr, &status);

  naxis[1] = naxis1;
  naxis[2] = naxis2;
}

void open_fitsfile(char *in_filename, int naxis1, int naxis2, double data[])
{
  int status = 0, anynull;
  long group = 0;
  double nulval = 0.0;
  fitsfile *fptr;

  fits_open_file(&fptr, in_filename, READONLY, &status);

  fits_read_2d_dbl(fptr, group, nulval, naxis2, naxis2, naxis1, data,
    &anynull, &status);
  fits_close_file(fptr, &status);
}

int read_header(const char *in_filename, const char *header_keyword, char *char_return)
{
  int count, status = 0, nkeys;
  const int char_length = 4096;
  char card[char_length], tmp_keyword[char_length];
  fitsfile *fptr;

  fits_open_file(&fptr, in_filename, READONLY, &status);
  fits_get_hdrspace(fptr, &nkeys, NULL, &status);

  for(count = 0; count < nkeys; count++)  { 
    fits_read_record(fptr, count, card, &status); // read keyword 
//    cout << card << "\n";

    strncpy(tmp_keyword, card, 8);
    tmp_keyword[8] = '\0';
//    if(count == 13) cout << tmp_keyword << "\n";
    if(strcmp(header_keyword, tmp_keyword) == 0) {
      strcpy(char_return, card);
      break;
    }

  }

  fits_close_file(fptr, &status);

// print any error messages
  if(status) fits_report_error(stderr, status);
  return status;
}

void create_fitsfile(char *h_filename, char *out_filename, int naxis1, int naxis2, double data[])
{
  int status1 = 0, status2 = 0;
  long fpixel[2] = {1, 1};
  long nelements = naxis1 * naxis2;
  char tmp1[] = "NAXIS1", tmp2[] = "NAXIS2", tmp_axis[] = "Axis length";
  fitsfile *fptr, *fptr2;

  fits_open_file(&fptr2, h_filename, READONLY, &status2);

  fits_create_file(&fptr, out_filename, &status1);
  fits_copy_header(fptr2, fptr, &status2);
  fits_update_key_lng(fptr, tmp1, naxis1, tmp_axis, &status1);
  fits_update_key_lng(fptr, tmp2, naxis2, tmp_axis, &status1);
  fits_write_pix(fptr, TDOUBLE, fpixel, nelements, data, &status1);
  fits_close_file(fptr, &status1);

  fits_close_file(fptr2, &status2);
}

void create_fitsfile(char *out_filename, int naxis1, int naxis2, double data[])
{
  int status1 = 0;
  long fpixel[2] = {1, 1};
  long nelements = naxis1 * naxis2;
  char tmp1[] = "NAXIS1", tmp2[] = "NAXIS2", tmp3[] = "Axis length";
  fitsfile *fptr;

  long naxis[2] = {naxis1, naxis2};

  fits_create_file(&fptr, out_filename, &status1);
  fits_create_img(fptr, DOUBLE_IMG, 2, naxis, &status1);
  fits_update_key_lng(fptr, tmp1, naxis1, tmp3, &status1);
  fits_update_key_lng(fptr, tmp2, naxis2, tmp3, &status1);
  fits_write_pix(fptr, TDOUBLE, fpixel, nelements, data, &status1);
  fits_close_file(fptr, &status1);
}

void create_3d_fitsfile(char *h_filename, char *out_filename, int naxis1, int naxis2, int naxis3, double data[])
{
  int status1 = 0, status2 = 0;
  long fpixel[3] = {1, 1, 1};
  long nelements = naxis1 * naxis2 * naxis3;
  char tmp_naxis[] = "NAXIS", tmp1[] = "NAXIS1", tmp2[] = "NAXIS2", tmp3[] = "NAXIS3", tmp_axis[] = "Axis length";
  fitsfile *fptr, *fptr2;

  fits_open_file(&fptr2, h_filename, READONLY, &status2);

  fits_create_file(&fptr, out_filename, &status1);
  fits_copy_header(fptr2, fptr, &status2);
  fits_update_key_lng(fptr, tmp_naxis, 3, tmp_axis, &status1);
  fits_update_key_lng(fptr, tmp1, naxis1, tmp_axis, &status1);
  fits_update_key_lng(fptr, tmp2, naxis2, tmp_axis, &status1);
  fits_write_key_lng(fptr, tmp3, naxis3, tmp_axis, &status1);
  fits_write_pix(fptr, TDOUBLE, fpixel, nelements, data, &status1);
  fits_close_file(fptr, &status1);

  fits_close_file(fptr2, &status2);
}

void create_3d_fitsfile(char *out_filename, int naxis1, int naxis2, int naxis3, double data[])
{
  int status1 = 0;
  long fpixel[3] = {1, 1, 1};
  long nelements = naxis1 * naxis2 * naxis3;
  char tmp_naxis[] = "NAXIS", tmp1[] = "NAXIS1", tmp2[] = "NAXIS2", tmp3[] = "NAXIS3", tmp_axis[] = "Axis length";
  fitsfile *fptr;

  long naxis[3] = {naxis1, naxis2, naxis3};

  fits_create_file(&fptr, out_filename, &status1);
  fits_create_img(fptr, DOUBLE_IMG, 3, naxis, &status1);
  fits_update_key_lng(fptr, tmp_naxis, 3, tmp_axis, &status1);
  fits_update_key_lng(fptr, tmp1, naxis1, tmp_axis, &status1);
  fits_update_key_lng(fptr, tmp2, naxis2, tmp_axis, &status1);
  fits_update_key_lng(fptr, tmp3, naxis3, tmp_axis, &status1);
  fits_write_pix(fptr, TDOUBLE, fpixel, nelements, data, &status1);
  fits_close_file(fptr, &status1);
}

int delete_header_keyword(const char *in_filename, const char *header_keyword)
{
//  int count, status = 0, nkeys;
  int status = 0;
  fitsfile *fptr;

  fits_open_file(&fptr, in_filename, READWRITE, &status);
//  fits_get_hdrspace(fptr, &nkeys, NULL, &status);

  fits_delete_key(fptr, header_keyword, &status);

  fits_close_file(fptr, &status);

// print any error messages
  if(status) fits_report_error(stderr, status);
  return status;
}

