#include "FiberCenter1.h"

int main(int argc, char *argv[])
{
  if (argc != 3) {
    cout << "The number of argments does not match.\n";
    cout << "  Usage: FiberCenter1 input_fits bias_value.\n";
    return 1;
  }

  int count1, count2, pos_x, pos_y, peak_x = 0, peak_y = 0;
  int offset_x[7] = {-1, 1, -2, 0, 2, -1, 1}, offset_y[7] = {2, 2, 0, 0, 0, -2, -2};
  double *get_data, bias = atof(argv[2]), peak_flux = 0, tmp_flux, gravity_x = 0, gravity_y = 0, total_flux = 0;
  vector<double> obs_data;
  char obs_frame[char_length];
  
  get_data = new double [naxis1_in * naxis2_in];
  if (!get_data) {
    cout << "Memory error.\n";
    return 2;
  }

/* open input fits file */

  strcpy (obs_frame, argv[1]);

  open_fitsfile(obs_frame, naxis1_in, naxis2_in, get_data);
  for (count1 = 0; count1 < naxis1_in * naxis2_in; count1++) {
    obs_data.push_back(get_data[count1] - bias);
    }

  delete [] get_data;

/* find flux peak */

  for(count2 = 1; count2 <= 7; count2++) {
    for(count1 = 1; count1 < 7 + count2; count1++) {
      pos_x = count1 * 2 - count2 - 7;
      pos_y = 14 - count2 * 2;

      tmp_flux = obs_data[(pos_x + 12) + (pos_y + 12) * naxis2_in];
      
      if(tmp_flux > peak_flux) {
        peak_flux = tmp_flux;
        peak_x = pos_x;
        peak_y = pos_y;
      }
    }
  }

  for(count2 = 1; count2 <= 6; count2++) {
    for(count1 = 1; count1 < 14 - count2; count1++) {
      pos_x = count1 * 2 + count2 - 14;
      pos_y = - count2 * 2;

      tmp_flux = obs_data[(pos_x + 12) + (pos_y + 12) * naxis2_in];
      
      if(tmp_flux > peak_flux) {
        peak_flux = tmp_flux;
        peak_x = pos_x;
        peak_y = pos_y;
      }
    }
  }

/* make sure that the object is not located near the edge of fiberbundle */

  if(abs(peak_y) == 12 || abs(2 * peak_x + peak_y) == 24 || abs(2 * peak_x - peak_y) == 24) {
  cout << "The object is located near the fiberbundle edge.\nMove the object to the center.\n";
  return 0;
  }

/* calculate the gravity center around the peak flux fiber */

  for(count1 = 0; count1 < 7; count1++) {
    tmp_flux = obs_data[(peak_x + offset_x[count1] + 12) + (peak_y + offset_y[count1] + 12) * naxis2_in];

    total_flux += tmp_flux;
    gravity_x += (double)offset_x[count1] * tmp_flux;
    gravity_y += (double)offset_y[count1] * tmp_flux;
  }

/* output */

  cout.precision(3);
  cout << "center (x, y) = (" << fixed << (peak_x + gravity_x / total_flux) / 2 << ", " << fixed << (peak_y + gravity_y / total_flux) / 2 << ")\n";

  return 0;
}
