#include "FiberCenter2.h"

int main(int argc, char *argv[])
{
  if (argc != 4) {
    cout << "The number of argments does not match.\n";
    cout << "  Usage: FiberCenter2 input_fits bias_value lib_file.\n";
    return 1;
  }

  int count1, count2, count3, count4, pos_x, pos_y, peak_x = 0, peak_y = 0;
  int pos_x_best = 0, pos_y_best = 0, FWHM_best = 0;
  int FWHM_max = 18;
  double *get_data, bias = atof(argv[2]), peak_flux = 0, tmp_flux;
  double norm, norm_best = 0, residual = 0, resi_best = 100;
  double flux_template[7][11][11][FWHM_max];
  vector<double> obs_data, obs_data_fit;
  char obs_frame[char_length], template_file[char_length];
//  char obs_frame[char_length], template_file[char_length];
  
  get_data = new double [naxis1_in * naxis2_in];
  if (!get_data) {
    cout << "Memory error.\n";
    return 2;
  }

/* open input fits file */

  strcpy(obs_frame, argv[1]);

  ifstream fin2(obs_frame);
  if (!fin2) {
    cout << "Cannot open " << obs_frame << ".\n";
    return 1;
  }

  open_fitsfile(obs_frame, naxis1_in, naxis2_in, get_data);
  for (count1 = 0; count1 < naxis1_in * naxis2_in; count1++) {
    obs_data.push_back(get_data[count1] - bias);
  }

  fin2.close();
  delete [] get_data;

/* open template file */

  strcpy(template_file, argv[3]);

  ifstream fin1(template_file);
  if(!fin1) {
    cout << "Cannot open " << template_file << ".\n";
    return 1;
  }

  for(count3 = 0; count3 < FWHM_max; count3++) {
    for(count4 = 0; count4 < 7; count4++) {
      for(count2 = 0; count2 < 11; count2++) {
        for(count1 = 0; count1 < 11; count1++) {
          fin1 >> flux_template[count4][count1][count2][count3];
        }
      }
    }
  }

  fin1.close();

/* find flux peak */

  for(count2 = 1; count2 <= 7; count2++) {
    for(count1 = 1; count1 < 7 + count2; count1++) {
      pos_x = count1 * 2 - count2 - 7;
      pos_y = 14 - count2 * 2;

      tmp_flux = obs_data[(pos_x + 12) + (pos_y + 12) * naxis2_in];
      
      if(tmp_flux > peak_flux) {
        peak_flux = tmp_flux;
        peak_x = pos_x;
        peak_y = pos_y;
      }
    }
  }

  for(count2 = 1; count2 <= 6; count2++) {
    for(count1 = 1; count1 < 14 - count2; count1++) {
      pos_x = count1 * 2 + count2 - 14;
      pos_y = - count2 * 2;

      tmp_flux = obs_data[(pos_x + 12) + (pos_y + 12) * naxis2_in];
      
      if(tmp_flux > peak_flux) {
        peak_flux = tmp_flux;
        peak_x = pos_x;
        peak_y = pos_y;
      }
    }
  }

/* make sure that the object is not located near the edge of fiberbundle */

  if(abs(peak_y) == 12 || abs(2 * peak_x + peak_y) == 24 || abs(2 * peak_x - peak_y) == 24) {
  cout << "The object is located near the fiberbundle edge.\nMove the object to the center.\n";
  return 0;
  }

/* extract fiber fluxes around the peak flux fiber */

  for(count4 = 0; count4 < 7; count4++) {
    obs_data_fit.push_back(obs_data[(peak_x + offset_x[count4]) + 12 + (peak_y + offset_y[count4] + 12) * naxis2_in]);
  }

/* calculate the residuals between observed frame and template */

  for(count3 = 0; count3 < FWHM_max; count3++) {
    for(count2 = 0; count2 < 11; count2++) {
      for(count1 = 0; count1 < 11; count1++) {

        norm = flux_template[3][count1][count2][count3] / peak_flux;

        for(count4 = 0; count4 < 7; count4++) {
          if(count4 == 3) continue;
          residual += func_square(norm * obs_data_fit[count4] - flux_template[count4][count1][count2][count3]);
        }

        if (residual < resi_best) {
          resi_best = residual;
          pos_x_best = count1;
          pos_y_best = count2;
          FWHM_best = count3;
          norm_best = norm;
        }
      residual = 0;
      }
    }
  }

/* output */

  cout << "# (x, y, FWHM, total flux, residual) = (";
  cout.precision(1);
  cout << fixed << (double)peak_x / 2 + (pos_x_best - 5) * 0.1 << ", " << fixed << (double)peak_y / 2 + (pos_y_best - 5) * 0.1 << ", ";
  cout << fixed << (double)FWHM_best * 0.1 + 0.3 << ", ";
  cout.precision(4);
  cout << scientific << 1 / norm_best << ", ";
  cout << scientific << resi_best << ")\n";

  return 0;
}
