#include "FiberCombine.h"

double calc_area(const double diff_x, const double diff_y, const double box_size, const int n_pitch)
{
  int count1, count2;
  double area = 0, tmp_x, tmp_y;

  for(count1 = 0; count1 < n_pitch; count1++) {
    tmp_x = diff_x + ((double)count1 + 0.5) / n_pitch * box_size;
    for(count2 = 0; count2 < n_pitch; count2++) {
      tmp_y = diff_y + ((double)count2 + 0.5) / n_pitch * box_size;

      if(r_core2 > func_square(tmp_x) + func_square(tmp_y)) area += 1.0;
    }
  }

  return area / func_square(n_pitch);
}

double get_obs_time(const char *obs_frame)
{
  int obs_time_sec, hour_int, tmp_int;
  const int time_2000_sec = 946728000;
  double exp_time, obs_time_century, utc_hour, gsmt_hour, tmp_double;
  char cmd[char_length], get_char[char_length], date_char[char_length], time_char[char_length] = "", hour_char[char_length], tmp_char[char_length] = "", hour_flag[char_length] = "false", *p_char;
  FILE *fp;

  tmp_int = read_header(obs_frame, "DATE-OBS", get_char);
  if(tmp_int) {
    cout << "fits file read error.\n";
    return 2;
  }

  p_char = get_char;
  strncpy(date_char, p_char + 11, 10);
  strcat(date_char, " ");

  tmp_int = read_header(obs_frame, "UT-STR  ", get_char);
  if(tmp_int) {
    cout << "fits file read error.\n";
    return 2;
  }

  p_char = get_char;
  strncpy(time_char, p_char + 11, 8);
//  cout << "obs_frame = " << obs_frame << " time_char = " << time_char << "\n";
// if hour >= 24, modify time_char and make flag on
  strncpy(hour_char, time_char, 2);
  hour_char[2] = '\0';
  hour_int = atoi(hour_char);
//  cout << "hour_int = " << hour_int << "\n";
  if(hour_int >= 24) {
    p_char = time_char;
    strncpy(tmp_char, p_char + 2, 2);
    
    sprintf(hour_char, "%02d", hour_int - 24);
    time_char[0] = hour_char[0];
    time_char[1] = hour_char[1];
    strcpy(hour_flag, "true");
  }

  strcat(date_char, time_char);
//  cout << "obs_frame = " << obs_frame << " date_char = " << date_char << "\n";

  tmp_int = read_header(obs_frame, "EXPTIME ", get_char);
  if(tmp_int) {
    cout << "fits file read error.\n";
    return 2;
  }

  p_char = get_char;
  strncpy(tmp_char, p_char + 11, 21);
  exp_time = atof(tmp_char);
//  cout << "obs_frame = " << obs_frame << " exp_time = " << exp_time << "\n";

  sprintf(cmd, "date -u -d '%s UT' +%%s", date_char);
  fp = popen(cmd, "r");
  fgets(get_char, sizeof(get_char), fp);
  pclose(fp);

  obs_time_sec = atoi(get_char) + exp_time / 2 - time_2000_sec;
  if(strcmp(hour_flag, "true") == 0) obs_time_sec += 86400;
//  if(strcmp(hour_flag, "true") == 0) {
//    cout << "hour_flag = " << hour_flag << "\n";
//    obs_time_sec += 86400;
//  }

//  cout.precision(16);
//  cout << "JD = " << (double)obs_time_sec / 86400 + 2451545 << "\n";

  utc_hour = (double)obs_time_sec / 3600 - floor(obs_time_sec / 86400) * 24 + 12;
//  utc_hour = (double)obs_time_sec / 3600 - floor(obs_time_sec / 86400) * 24;
  obs_time_century = (double)obs_time_sec / 86400 / 36525;
//  cout.precision(16);
//cout << "obs_time_century = " << obs_time_century << "\n";
//  cout << "obs_frame = " << obs_frame << " utc_hour = " << utc_hour << "\n";

  tmp_double = 6.697374558 + 8640184.812866 / 3600 * obs_time_century + 0.093104 / 3600 * pow(obs_time_century, 2) - 0.0000062 / 3600 * pow(obs_time_century, 3) + 1.0027379094 * utc_hour;
  gsmt_hour = tmp_double - floor(tmp_double / 24) * 24;

//cout << "gsmt_0_hour = " << 6.697374558 + 8640184.812866 / 3600 * obs_time_century + 0.093104 / 3600 * pow(obs_time_century, 2) - 0.0000062 / 3600 * pow(obs_time_century, 3) << "\n";

//  return 6.697374558 + (8640184.812866 * obs_time_century + 0.093104 * pow(obs_time_century, 2) - 0.0000062 * pow(obs_time_century, 3)) / 3600 + 1.0027379094 * utc_hour;
  return gsmt_hour;
}

void get_lambda_info(const char *obs_frame, double *lambda_0, double *lambda_slope)
{
  int tmp_int;
  char get_char[char_length], tmp_char[char_length] = "", *p_char;
  
  tmp_int = read_header(obs_frame, "CRVAL1  ", get_char);
  if(tmp_int) {
    cout << "fits file read error.\n";
  }

  p_char = get_char;
  strncpy(tmp_char, p_char + 11, 21);
  *lambda_0 = atof(tmp_char);

  tmp_int = read_header(obs_frame, "CD1_1   ", get_char);
  if(tmp_int) {
    cout << "fits file read error.\n";
  }

  p_char = get_char;
  strncpy(tmp_char, p_char + 11, 21);
  *lambda_slope = atof(tmp_char);
}

double calc_air_refrac(const double lambda, const double zenith_distance)
{
  const double refrac_index_1 = (5792105 / (238.0185 - 1 / func_square(lambda * 1e-4)) + 167917 / (57.362 - 1 / func_square(lambda * 1e-4))) * 1e-8 * air_pressure / 1013;
  const double refrac_R0 = refrac_index_1 * (1 - scale_height);
  const double refrac_R1 = func_square(refrac_index_1) * 0.5 - refrac_index_1 * scale_height;

//  cout.precision(10);
//  cout << "lambda = " << lambda << " refrac_index = " << refrac_index_1 + 1 << "\n";
//  cout << "R0, R1, zenith distance  = " << refrac_R0 << ", " << refrac_R1 << ", " << zenith_distance << "\n";

  return (refrac_R0 * tan(zenith_distance * degree_to_radian) + refrac_R1 * pow(tan(zenith_distance * degree_to_radian), 3)) * 206265;
}

struct st_frame {
  double offset_x, offset_y, weight, obs_time;
  vector<int> mask;
  vector<double> flux, azi_elev;
};

int main(int argc, char *argv[])
{
/* initialize */

  if (argc != 6) {
    cout << "The number of argments does not match.\n";
    cout << "  Usage: FiberCombine input_fits_list output_name input_offset_weight param_file input_mask_fits.\n";
    return 1;
  }

  int count1, count2, count3, count4, count5, naxis_in[3], naxis_out[4], num_frame, n_image_out[4], n_pitch, n_sigclip, tmp_int, tmp_int2, tmp_int3;
//  int count1, count2, count3, count4, count5, naxis_in[3], naxis_out[3], num_frame, n_image_out[3], n_pitch, tmp_int, tmp_int2;
  double *get_data, *out_data, *out_3d_data, magnify, image_center_x, image_center_y, tmp_center_x, tmp_center_y, sum_flux, sum_area, thresh_sigclip, tmp_double, obj_RA_Dec[2], long_lat[2], lambda_0, lambda_slope, lambda_fiducial, air_refrac_diff[3];

  vector<double> v_sigma, common_mask, v_tmp;
  vector<st_frame> frame;
  vector<double>::iterator it_v;
  vector< vector<double> > v_flux, v_area;
  vector< vector<double> >::iterator it_sum_flux, it_sum_area;

  char obs_frame[char_length], out_frame[char_length], tmp_char[char_length];
  string str_tmp, str_comb = "";
  stringstream ss_tmp;
  vector<string> input_file_name;
  string::size_type comment_start = 0;

/* input files */  

// read parameter file

  ifstream fin3(argv[4]);
  if(!fin3) {
    cout << "Cannot open " << argv[4] << ".\n";
    return 1;
  }

  while(getline(fin3, str_tmp)) {
    if((comment_start = str_tmp.find('#')) != string::size_type(-1)) str_tmp = str_tmp.substr(0, comment_start);
    if(str_tmp.empty()) continue;

    str_comb += str_tmp;
  }
  fin3.close();

  strcpy(tmp_char, str_comb.c_str());
  sscanf(tmp_char, "%d %d %lf %d %d %lf %lf %lf %lf %lf %lf", &n_image_out[1], &n_image_out[2], &magnify, &n_pitch, &n_sigclip, &thresh_sigclip, &lambda_fiducial, &obj_RA_Dec[0], &obj_RA_Dec[1], &long_lat[0], &long_lat[1]);

// read input file list

  ifstream fin1(argv[1]);
  if(!fin1) {
    cout << "Cannot open " << argv[1] << ".\n";
    return 1;
  }

  while(getline(fin1, str_tmp)) {
    if((comment_start = str_tmp.find('#')) != string::size_type(-1)) str_tmp = str_tmp.substr(0, comment_start);
    if(str_tmp.empty()) continue;

    input_file_name.push_back(str_tmp);
  }
  fin1.close();

// input fits files
//// object frames

  num_frame = input_file_name.size();
  frame.resize(num_frame);
//  st_frame frame[num_frame];

  strcpy(obs_frame, input_file_name[0].c_str());
  get_naxis(obs_frame, naxis_in);
  get_lambda_info(obs_frame, &lambda_0, &lambda_slope);

  get_data = new double [naxis_in[1] * naxis_in[2]];
  if(!get_data) {
    cout << "Memory error.\n";
    return 2;
  }

  for(count1 = 0; count1 < num_frame; count1++) {
    strcpy(obs_frame, input_file_name[count1].c_str());

    frame[count1].obs_time = get_obs_time(obs_frame);
//    cout << "count1 = " << count1 << " obs_time = " << frame[count1].obs_time << "\n";
    calc_azi_elev(frame[count1].obs_time, obj_RA_Dec, long_lat, frame[count1].azi_elev);
//    cout << "count1 = " << count1 << " azimuth = " << frame[count1].azi_elev[0] << " elevation = " << frame[count1].azi_elev[1] << "\n";

    frame[count1].flux.reserve(naxis_in[1] * naxis_in[2]);
    open_fitsfile(obs_frame, naxis_in[1], naxis_in[2], get_data);
    for(count2 = 0; count2 < naxis_in[1] * naxis_in[2]; count2++) {
      frame[count1].flux[count2] = get_data[count2];
    }
    frame[count1].mask.resize(naxis_in[1] * naxis_in[2], 0);
  }

//// mask file
  open_fitsfile(argv[5], naxis_in[1], naxis_in[2], get_data);
  for(count1 = 0; count1 < naxis_in[1] * naxis_in[2]; count1++) {
    common_mask.push_back(get_data[count1]);
  }

  delete [] get_data;

// read offset file

  ifstream fin2(argv[3]);
  if(!fin2) {
    cout << "Cannot open " << argv[3] << ".\n";
    return 1;
  }

  count1 = 0;
  while(getline(fin2, str_tmp)) {
    if((comment_start = str_tmp.find('#')) != string::size_type(-1)) str_tmp = str_tmp.substr(0, comment_start);
    if(str_tmp.empty()) continue;

    strcpy(tmp_char, str_tmp.c_str());
    sscanf(tmp_char, "%lf %lf %lf", &frame[count1].offset_x, &frame[count1].offset_y, &frame[count1].weight);

    count1++;
    if(count1 >= num_frame) break;
  }
  fin2.close();

/* combine frames */

// initialize

  out_data = new double [naxis_in[1] * n_image_out[1] * n_image_out[2]];
  if(!out_data) {
    cout << "Memory error.\n";
    return 2;
  }

//  it_out_data_tmp = out_data_tmp.begin();
//  out_data_tmp.insert(it_out_data_tmp, naxis_in[1] * n_image_out[1] * n_image_out[2], 0);

  image_center_x = n_image_out[1] / 2;
  image_center_y = n_image_out[2] / 2;

// wavelength loop
  for(count1 = 0; count1 < naxis_in[1]; count1++) {
cout << "count1 = " << count1 << "\r";

// reset total flux and total area
    v_flux.clear();
    v_area.clear();
    v_flux.resize(n_image_out[1] * n_image_out[2], v_tmp);
    v_area.resize(n_image_out[1] * n_image_out[2], v_tmp);
//    it_sum_flux = sum_flux.begin();
//    it_sum_area = sum_area.begin();
//    sum_flux.insert(it_sum_flux, n_image_out[1] * n_image_out[2], 0);
//    sum_area.insert(it_sum_area, n_image_out[1] * n_image_out[2], 0);

// frame loop
    for(count2 = 0; count2 < num_frame; count2++) {

//// calculate air refraction difference
      air_refrac_diff[0] = arcsec_to_fiber * (calc_air_refrac(lambda_0 + count1 * lambda_slope, 90 - frame[count2].azi_elev[1]) - calc_air_refrac(lambda_fiducial, 90 - frame[count2].azi_elev[1]));
      air_refrac_diff[1] = air_refrac_diff[0] * cos((pa_fiber + 90 - frame[count2].azi_elev[0]) * degree_to_radian);
      air_refrac_diff[2] = air_refrac_diff[0] * sin((pa_fiber + 90 - frame[count2].azi_elev[0]) * degree_to_radian);
//      air_refrac_diff[1] = 0;
//      air_refrac_diff[2] = 0;
//      if(count1 == 100) cout << "azimuth = " << frame[count2].azi_elev[0] << " elevation = " << frame[count2].azi_elev[1] << "\n";
//      if(count1 == 100) cout << "air_refrac_diff_0 = " << air_refrac_diff[0] << "\n";
//      if(count1 == 100) cout << "air_refrac_diff (x, y) = (" << air_refrac_diff[1] << ", " << air_refrac_diff[2] << ")\n";

// fiber loop
      for(count3 = 0; count3 < num_fiber; count3++) {
//	tmp_center_x = image_center_x + (fiber_pos_x[count3] - fiber_center_x - frame[count2].offset_x);
//	tmp_center_y = image_center_y + (fiber_pos_y[count3] - fiber_center_y - frame[count2].offset_y) * sqrt3_div2;
	tmp_center_x = image_center_x * magnify + ((fiber_pos_x[count3] - fiber_center_x) / 2 - frame[count2].offset_x) + air_refrac_diff[1];
	tmp_center_y = image_center_y * magnify + ((fiber_pos_y[count3] - fiber_center_y) / 2 - frame[count2].offset_y) * sqrt3_div2 + air_refrac_diff[2];

//if(count1 == 0) cout << "tmp_center (x, y) = (" << tmp_center_x << ", " << tmp_center_y << ")\n";

// image loop
	for(count4 = 0; count4 < n_image_out[1]; count4++) {
	  for(count5 = 0; count5 < n_image_out[2]; count5++) {
//	    if(frame[count2].flux[count1 + count3 * naxis_in[1]] == 0) continue;
	    if(common_mask[count1 + count3 * naxis_in[1]] == 1 || frame[count2].mask[count1 + count3 * naxis_in[1]] == 1) continue;

	    if(tmp_center_x - r_core > (count4 + 1) * magnify || tmp_center_x + r_core <= count4 * magnify || tmp_center_y - r_core > (count5 + 1) * magnify || tmp_center_y + r_core <= count5 * magnify) continue;

	    tmp_double = calc_area(count4 * magnify - tmp_center_x, count5 * magnify - tmp_center_y, magnify, n_pitch);
	    if(tmp_double == 0) continue;

	    v_area[count4 + count5 * n_image_out[1]].push_back(tmp_double * frame[count2].weight);
	    v_flux[count4 + count5 * n_image_out[1]].push_back(frame[count2].flux[count1 + count3 * naxis_in[1]]);

//	    v_area[count4 + count5 * n_image_out[1]].push_back(calc_area(count4 * magnify - tmp_center_x, count5 * magnify - tmp_center_y, magnify, n_pitch) * frame[count2].weight);

//	      tmp = calc_area(count4 * magnify - tmp_center_x, count5 * magnify - tmp_center_y, magnify, n_pitch);
//	      sum_flux[count4 + count5 * n_image_out[1]] += frame[count2].flux[count1 + count3 * naxis_in[1]] * tmp;
//	      sum_area[count4 + count5 * n_image_out[1]] += tmp;

	  }
	}

      }
    }

// output

    for(count2 = 0; count2 < n_image_out[1]; count2++) {
      for(count3 = 0; count3 < n_image_out[2]; count3++) {
	tmp_int = count1 + count2 * naxis_in[1] + count3 * naxis_in[1] * n_image_out[1];
	tmp_int2 = count2 + count3 * n_image_out[1];

//        if(count1 == 103) cout << "v_area.size = " << v_area[tmp_int2].size() << "\n";

        if((int)v_area[tmp_int2].size() == 0) continue;

// sigclip
	for(count4 = 0; count4 < n_sigclip; count4++) {
	  v_sigma.resize(v_flux[tmp_int2].size());
	  if(v_flux[tmp_int2].size() < 3) break;

	  func_v_sigma(v_flux[tmp_int2], v_sigma);

	  tmp_int3 = 0;
	  for(count5 = 1; count5 < (int)v_sigma.size(); count5++) {
	    if(v_sigma[count5] > tmp_int3) tmp_int3 = count5;
	  }

	  if(v_sigma[tmp_int3] > thresh_sigclip) {
	    it_v = v_flux[tmp_int2].begin();
	    it_v += tmp_int3;
	    v_flux[tmp_int2].erase(it_v);

	    it_v = v_area[tmp_int2].begin();
	    it_v += tmp_int3;
	    v_area[tmp_int2].erase(it_v);
	  }
	  else break;

/*
	  cout << "sigma =";
	  for (count5 = 0; count5 < (int)v_area[tmp_int2].size(); count5++) {
	    cout << " " << v_sigma[count5];
	  }
	  cout << "\n";
*/
	}

	sum_flux = 0;
	sum_area = 0;
	for (count4 = 0; count4 < (int)v_area[tmp_int2].size(); count4++) {
	  sum_flux += v_flux[tmp_int2][count4] * v_area[tmp_int2][count4];
	  sum_area += v_area[tmp_int2][count4];
	}

	if (sum_area == 0) {
	  out_data[tmp_int] = 0;
	}
	else {
	  out_data[tmp_int] = sum_flux / sum_area;
//	  out_data[tmp_int] = sum_flux[count2 + count3 * n_image_out[1]] / tmp;
//cout << "x = " << count1 << " y = " << count2 + count3 * n_image_out[1] << " out_data = " << out_data[tmp_int] << "\n";
	}

      }
    }
  }

/* output file */

  strcpy(out_frame, argv[2]);
  naxis_out[1] = naxis_in[1];
  naxis_out[2] = n_image_out[1] * n_image_out[2];
//  naxis_out[1] = n_image_out[1];
//  naxis_out[2] = n_image_out[2];

// cut zero regions

/*
  for (count1 = 0; count1 < naxis_out[1] * naxis_out[2]; count1++) {
    if (out_data_tmp[count1] != 0) {
      cut_lower = count1;
      break;
    }
  }

  for (count1 = naxis_out[1] * naxis_out[2] - 1; count1 > 0; count1--) {
    if (out_data_tmp[count1] != 0) {
      cut_upper = count1;
      break;
    }
  }

  it_out_data_tmp = out_data_tmp.begin();
  tmp_int = int(cut_upper / (naxis_in[1] * n_image_out[1])) + 1;
  cout << "cut region (upper): " << tmp_int << "\n";
  out_data_tmp.erase(it_out_data_tmp + tmp_int * naxis_in[1] * n_image_out[1], it_out_data_tmp + naxis_in[1] * n_image_out[1] * n_image_out[2]);
  n_image_out[2] -= tmp_int;

  it_out_data_tmp = out_data_tmp.begin();
  tmp_int = int(cut_lower / (naxis_in[1] * n_image_out[1]));
  cout << "cut region (lower): " << tmp_int << "\n";
  out_data_tmp.erase(it_out_data_tmp, it_out_data_tmp + tmp_int * naxis_in[1] * n_image_out[1]);
  n_image_out[2] -= tmp_int;

// put data in out_data
  out_data = new double [naxis_in[1] * n_image_out[1] * n_image_out[2]];
  if (!out_data) {
    cout << "Memory error.\n";
    return 2;
  }

  for (count1 = 0; count1 < naxis_in[1] * n_image_out[1] * n_image_out[2]; count1++) {
    out_data[count1] = out_data_tmp[count1];
  }
*/

// output fits file
  remove(out_frame);
  create_fitsfile(obs_frame, out_frame, naxis_out[1], naxis_out[2], out_data);

//delete header keyword
  delete_header_keyword(out_frame, "WAT0_001");
  delete_header_keyword(out_frame, "WAT1_001");
  delete_header_keyword(out_frame, "WAT2_001");

/* make 3d fits file */

//  strcpy(out_frame, "tmp_3d.fits");
  tmp_int = strlen(out_frame);
  out_frame[tmp_int - 5] = '\0';
  strcat(out_frame, "-3d.fits");

  remove(out_frame);
  naxis_out[1] = n_image_out[1];
  naxis_out[2] = n_image_out[2];
  naxis_out[3] = naxis_in[1];

  out_3d_data = new double [naxis_in[1] * n_image_out[1] * n_image_out[2]];
  if(!out_3d_data) {
    cout << "Memory error.\n";
    return 2;
  }

  for(count1 = 0; count1 < naxis_out[1]; count1++) {
    for(count2 = 0; count2 < naxis_out[2]; count2++) {
      for(count3 = 0; count3 < naxis_out[3]; count3++) {
	out_3d_data[count1 + count2 * naxis_out[1] + count3 * naxis_out[1] * naxis_out[2]] = out_data[count3 + count1 * naxis_in[1] + count2 * naxis_in[1] * n_image_out[1]];
      }
    }
  }

//  create_3d_fitsfile(obs_frame, out_frame, naxis_out[1], naxis_out[2], naxis_out[3], out_data);
  create_3d_fitsfile(out_frame, naxis_out[1], naxis_out[2], naxis_out[3], out_3d_data);

//  delete_header_keyword(out_frame, "WAT0_001");
//  delete_header_keyword(out_frame, "WAT1_001");
//  delete_header_keyword(out_frame, "WAT2_001");

  delete [] out_data;
  delete [] out_3d_data;
  return 0;
}

