#ifndef _FIBER_COMBINE_H

#define _FIBER_COMBINE_H

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <string>
#include <sstream>
#include <cstring>
#include <cmath>
#include <vector>
#include "fitsio.h"
using namespace std;

static const int char_length = 4096, num_fiber = 127, fiber_center_x = 13, fiber_center_y = 13;
static const double degree_to_radian = 3.14159265358979323 / 180, sqrt3_div2 = sqrt(3) / 2;
static const double r_core = 0.4, r_core2 = r_core * r_core, arcsec_to_fiber = 1 / 2.34, pa_fiber = 127.3, scale_height = 0.0130, air_pressure = 980;

static const int fiber_pos_x[num_fiber] = {16, 22, 18, 19, 8, 11, 14, 12, 8, 11, 15, 11, 11, 19, 15, 11, 15, 7, 8, 18, 11, 10, 9, 13, 16, 17, 14, 17, 9, 10, 13, 17, 19, 12, 16, 14, 13, 7, 15, 20, 17, 18, 19, 16, 14, 9, 22, 5, 6, 15, 19, 19, 12, 7, 17, 20, 21, 16, 25, 17, 13, 21, 19, 20, 24, 20, 21, 18, 23, 13, 16, 24, 23, 14, 22, 20, 18, 13, 23, 11, 15, 22, 21, 15, 17, 9, 7, 9, 5, 10, 10, 7, 6, 7, 21, 2, 3, 5, 1, 9, 12, 4, 6, 7, 10, 20, 5, 6, 3, 4, 8, 6, 5, 4, 8, 20, 4, 8, 9, 12, 13, 14, 3, 6, 12, 10, 2};
static const int fiber_pos_y[num_fiber] = {23, 19, 15, 25, 19, 5, 23, 19, 11, 21, 17, 13, 9, 17, 25, 25, 21, 9, 7, 19, 17, 3, 13, 21, 19, 13, 11, 25, 25, 15, 25, 17, 21, 15, 15, 19, 13, 21, 13, 23, 21, 23, 1, 3, 3, 1, 7, 5, 3, 1, 9, 5, 3, 1, 1, 7, 9, 7, 13, 5, 1, 21, 13, 15, 15, 3, 5, 11, 17, 9, 11, 11, 13, 7, 15, 3, 7, 5, 9, 1, 5, 11, 13, 9, 9, 9, 17, 17, 21, 23, 11, 13, 19, 5, 17, 11, 13, 13, 13, 5, 23, 19, 23, 25, 7, 19, 9, 11, 17, 7, 15, 7, 17, 15, 23, 11, 11, 3, 21, 11, 17, 15, 9, 15, 7, 19, 15};


void get_naxis(char *get_frame, int naxis[]);
void open_fitsfile(char *in_filename, int naxis1, int naxis2, double data[]);
int read_header(const char *in_filename, const char *header_keyword, char *char_return);
void create_fitsfile(char *h_filename, char *out_filename, int naxis1, int naxis2, double data[]);
void create_3d_fitsfile(char *out_filename, int naxis1, int naxis2, int naxis3, double data[]);
int delete_header_keyword(const char *in_filename, const char *header_keyword);

void calc_azi_elev(const double gmst_hour, const double obj_RA_Dec[], const double long_lat[], vector<double> &azi_elev);
void func_v_sigma(const vector<double> &v_input, vector<double> &v_output);
//double func_rotate_x(double input_x, double input_y, double center_x, double center_y, double rotate);
//double func_rotate_y(double input_x, double input_y, double center_x, double center_y, double rotate);
double func_square(double x);

#endif
