/*
program for detecting cosmic rays

detection argorithm: 
  1. detect a pixel whose value is larger than the threshold
  2. detect the extent of pixels polluted by cosmic rays in a line
  3. possible polluted pixels are commented out
*/

#include "FiberDetectCosmic.h"

int main(int argc, char *argv[])
{
  if (argc != 6) {
    cout << "The number of argments does not match.\n";
    cout << "Usage: FiberDetectCosmic input_fits input_mask_fits output_dat cr-thresh cr-thresh_sub\n";
    return 1;
  }

  int count1, count2, count3, naxis[3];
  int pos_fits_start, pos_fits_end, cr_x_start, cr_x_end, cr_y;
  double *get_data, *get_mask, cr_thresh = atof(argv[4]), cr_thresh_sub = atof(argv[5]), tmp;
  char obs_frame[char_length], mask_frame[char_length], cr_list[char_length];
  string continue_flag, comment_flag;
  vector<int> mask_data;
  vector<double> obs_data;

/* open input fits files */

  strcpy(obs_frame, argv[1]);
  strcpy(mask_frame, argv[2]);
  get_naxis(obs_frame, naxis);

  get_data = new double [naxis[1] * naxis[2]];
  if (!get_data) {
    cout << "Memory error.\n";
    return 2;
  }

  get_mask = new double [naxis[1] * naxis[2]];
  if (!get_mask) {
    cout << "Memory error.\n";
    return 2;
  }

  open_fitsfile(obs_frame, naxis[1], naxis[2], get_data);
  open_fitsfile(mask_frame, naxis[1], naxis[2], get_mask);
  for (count1 = 0; count1 < naxis[1] * naxis[2]; count1++) {
    obs_data.push_back(get_data[count1]);
    mask_data.push_back((int)get_mask[count1]);
  }

  delete [] get_data;
  delete [] get_mask;

/* prepare output file */

  strcpy(cr_list, argv[3]);
  remove(cr_list);
  ofstream fout(cr_list);
  if(!fout) {
    cout << "Cannot open file " << cr_list << ".\n";
    return 2;
  }

  fout << "# x_start x_end y\n";

/* main roop */

  for(count2 = 0; count2 < naxis[2]; count2++) {
    cr_y = count2;
    for(count1 = approx_buff; count1 < naxis[1] - approx_buff; count1++) {
      cr_x_start = count1; 
      cr_x_end = cr_x_start; 
      pos_fits_start = count1 + count2 * naxis[1];
      pos_fits_end = pos_fits_start;

      continue_flag = "false";
      comment_flag = "false";

// check whether the pixel value is larger than the threshold
      if(obs_data[pos_fits_start] < cr_thresh) continue;

// check the extent of pixels polluted by cosmic rays in a line
      while(obs_data[pos_fits_end + 1] >= cr_thresh && count1 < naxis[1] - approx_buff) {
        count1++;
	pos_fits_end++;
	cr_x_end++;
      }

      if(count1 >= naxis[1] - approx_buff) break;

// check whether the pixel_end locates around the masked pixel
      for(count3 = pos_fits_start - approx_buff; count3 <= pos_fits_end + approx_buff; count3++) {
	if(mask_data[count3] == 0) {
	  continue_flag = "true";
	  break;
	}
      }
      if(continue_flag == "true") continue;

// check whether the background-subtracted pixel value is larger than the threshold
      for(count3 = cr_x_start; count3 <= cr_x_end; count3++) {
	tmp = obs_data[count3 + cr_y * naxis[1]] - func_linear_approx((double)(cr_x_start - approx_buff), (double)(cr_x_end + approx_buff), obs_data[pos_fits_start - approx_buff], obs_data[pos_fits_end + approx_buff], (double)count3);
	if(tmp < cr_thresh_sub) {
	  comment_flag = "true";
	  break;
	}
      }

// comment out if comment_flag is true
      if(comment_flag == "true") fout << "#";

// output cosmic ray position
      fout << cr_x_start + 1 << " " << cr_x_end + 1 << " " << cr_y + 1 << "\n";
    }
  }

/* end script */

  fout.close();

  return 0;
}
