/*
program for detecting cosmic rays (ver. 2)

detection argorithm: 
  1. use raw frame devided by flat frame (input both frames)
  2. detect cosmic rays with the following process in x and y direction: detect a pixel whose value is larger than sigma_threshold of the previous and next 3 pixels
  3. iterate process 2
*/

#include "FiberDetectCosmic2.h"

void input_v_tmp_x(vector<double> &v_input, vector<double> &v_output, int x, int y, int naxis1)
{
  v_output[0] = v_input[x - 3 + y * naxis1];
  v_output[1] = v_input[x - 2 + y * naxis1];
  v_output[2] = v_input[x - 1 + y * naxis1];
  v_output[3] = v_input[x + 1 + y * naxis1];
  v_output[4] = v_input[x + 2 + y * naxis1];
  v_output[5] = v_input[x + 3 + y * naxis1];
}

void input_v_tmp_x2(vector<double> &v_input, vector<double> &v_output, int x, int y, int naxis1)
{
  v_output[0] = v_input[x - 3 + y * naxis1];
  v_output[1] = v_input[x - 2 + y * naxis1];
  v_output[2] = v_input[x - 1 + y * naxis1];
  v_output[3] = v_input[x + 2 + y * naxis1];
  v_output[4] = v_input[x + 3 + y * naxis1];
  v_output[5] = v_input[x + 4 + y * naxis1];
}

void input_v_tmp_y(vector<double> &v_input, vector<double> &v_output, int x, int y, int naxis1)
{
  v_output[0] = v_input[x + (y - 3) * naxis1];
  v_output[1] = v_input[x + (y - 2) * naxis1];
  v_output[2] = v_input[x + (y - 1) * naxis1];
  v_output[3] = v_input[x + (y + 1) * naxis1];
  v_output[4] = v_input[x + (y + 2) * naxis1];
  v_output[5] = v_input[x + (y + 3) * naxis1];
}

void input_v_tmp_y2(vector<double> &v_input, vector<double> &v_output, int x, int y, int naxis1)
{
  v_output[0] = v_input[x + (y - 3) * naxis1];
  v_output[1] = v_input[x + (y - 2) * naxis1];
  v_output[2] = v_input[x + (y - 1) * naxis1];
  v_output[3] = v_input[x + (y + 2) * naxis1];
  v_output[4] = v_input[x + (y + 3) * naxis1];
  v_output[5] = v_input[x + (y + 4) * naxis1];
}

void remove_cosmicrays(vector<double> &v_data_in, vector<double> &v_data_out, vector<int> &v_flag, int naxis[], int check_buff)
{
  int count1, count2, count3, pos_xy, cr_length;

  v_data_out = v_data_in;

  for(count2 = 0; count2 < naxis[2]; count2++) {
    for(count3 = check_buff; count3 < naxis[1] - check_buff; count3++) {
      pos_xy = count3 + count2 * naxis[1];
      if(v_flag[pos_xy] == 0) continue;

      cr_length = 1;
      while(v_flag[pos_xy + cr_length] == 1) {
	cr_length++;
      }

      for(count1 = count3; count1 < count3 + cr_length; count1++) {
	v_data_out[count1 + count2 * naxis[1]] = func_linear_approx((double)count3 - 1, (double)count3 + cr_length, v_data_in[count3 - 1 + count2 * naxis[1]], v_data_in[count3 + cr_length + count2 * naxis[1]], (double)count1);
      }

      count3 += cr_length - 1;
    }
  }
}

int main(int argc, char *argv[])
{
  if (argc != 7) {
    cout << "The number of argments does not match.\n";
    cout << "Usage: FiberDetectCosmic2 input_fits input_flat_fits output_fits cr_thresh_count cr_thresh_sigma cr_thresh_sigma2\n";
    return 1;
  }

  int count1, count2, count3, naxis[3], n_iterate = 100, pos_xy;
  double *get_data, *get_flat, cr_count = atof(argv[4]), cr_sigma = atof(argv[5]), cr_sigma2 = atof(argv[6]);
  char obs_frame[char_length], flat_frame[char_length], cr_list[char_length];
  string detect_flag;
  vector<int> cr_flag;
  vector<double> obs_data, obs_data_init, obs_flat, data_flat, v_tmp(6, 0);

/* open input fits files */

  strcpy(obs_frame, argv[1]);
  strcpy(flat_frame, argv[2]);
  get_naxis(obs_frame, naxis);

  get_data = new double [naxis[1] * naxis[2]];
  if (!get_data) {
    cout << "Memory error.\n";
    return 2;
  }

  get_flat = new double [naxis[1] * naxis[2]];
  if (!get_flat) {
    cout << "Memory error.\n";
    return 2;
  }

  open_fitsfile(obs_frame, naxis[1], naxis[2], get_data);
  open_fitsfile(flat_frame, naxis[1], naxis[2], get_flat);
  for (count1 = 0; count1 < naxis[1] * naxis[2]; count1++) {
    obs_data_init.push_back(get_data[count1]);
    obs_flat.push_back((int)get_flat[count1]);
    cr_flag.push_back(0);
  }

  delete [] get_data;
  delete [] get_flat;

/* prepare for main loop */

  obs_data = obs_data_init;
  data_flat.resize(naxis[1] * naxis[2], 0);
  cr_flag.resize(naxis[1] * naxis[2], 0);

/* main roop */

  for(count1 = 0; count1 < n_iterate; count1++) {
    detect_flag = "false";
// make data frame devided by flat frame
    for(count2 = 0; count2 < naxis[1] * naxis[2]; count2++) {
      if (obs_flat[count2] == 0) data_flat[count2] = 0;
      else data_flat[count2] = obs_data[count2] / obs_flat[count2];
    }

// check 1 pixel
// check in x direction
    for(count2 = 0; count2 < naxis[2]; count2++) {
      for(count3 = check_buff; count3 < naxis[1] - check_buff; count3++) {
	pos_xy = count3 + count2 * naxis[1];

	input_v_tmp_x(data_flat, v_tmp, count3, count2, naxis[1]);
	if(data_flat[pos_xy] - func_mean(v_tmp) > func_stddev(v_tmp) * cr_sigma) {
	  if(cr_flag[pos_xy] == 0) detect_flag = "true";
	  cr_flag[pos_xy] = 1;
	}
      }
    }

    remove_cosmicrays(obs_data_init, obs_data, cr_flag, naxis, check_buff);

// make data frame devided by flat frame
    for(count2 = 0; count2 < naxis[1] * naxis[2]; count2++) {
      if (obs_flat[count2] == 0) data_flat[count2] = 0;
      else data_flat[count2] = obs_data[count2] / obs_flat[count2];
    }

// check in y direction
    for(count3 = 0; count3 < naxis[1]; count3++) {
      for(count2 = check_buff; count2 < naxis[2] - check_buff; count2++) {
	pos_xy = count3 + count2 * naxis[1];

	input_v_tmp_y(data_flat, v_tmp, count3, count2, naxis[1]);
	if(data_flat[pos_xy] - func_mean(v_tmp) > func_stddev(v_tmp) * cr_sigma) {
	  if(cr_flag[pos_xy] == 0) detect_flag = "true";
	  cr_flag[pos_xy] = 1;
	}
      }
    }

    remove_cosmicrays(obs_data_init, obs_data, cr_flag, naxis, check_buff);

    if(detect_flag == "true") continue;

// check 2 pixels
// check in x direction
    for(count2 = 0; count2 < naxis[2] - 1; count2++) {
      for(count3 = check_buff; count3 < naxis[1] - (check_buff + 1); count3++) {
	pos_xy = count3 + count2 * naxis[1];

	input_v_tmp_x2(data_flat, v_tmp, count3, count2, naxis[1]);
	if(data_flat[pos_xy] - func_mean(v_tmp) > func_stddev(v_tmp) * cr_sigma2) {
	  if(cr_flag[pos_xy] == 0 || cr_flag[pos_xy + 1] == 0) detect_flag = "true";
	  cr_flag[pos_xy] = 1;
	  cr_flag[pos_xy + 1] = 1;
	}
      }
    }

    remove_cosmicrays(obs_data_init, obs_data, cr_flag, naxis, check_buff);

// make data frame devided by flat frame
    for(count2 = 0; count2 < naxis[1] * naxis[2]; count2++) {
      if (obs_flat[count2] == 0) data_flat[count2] = 0;
      else data_flat[count2] = obs_data[count2] / obs_flat[count2];
    }

// check in y direction
    for(count3 = 0; count3 < naxis[1]; count3++) {
      for(count2 = check_buff; count2 < naxis[2] - (check_buff + 1); count2++) {
	pos_xy = count3 + count2 * naxis[1];

	input_v_tmp_y2(data_flat, v_tmp, count3, count2, naxis[1]);
	if(data_flat[pos_xy] - func_mean(v_tmp) > func_stddev(v_tmp) * cr_sigma2) {
	  if(cr_flag[pos_xy] == 0 || cr_flag[pos_xy + naxis[1]] == 0) detect_flag = "true";
	  cr_flag[pos_xy] = 1;
	  cr_flag[pos_xy + naxis[1]] = 1;
	}
      }
    }

    remove_cosmicrays(obs_data_init, obs_data, cr_flag, naxis, check_buff);

    if(detect_flag == "false") break;
  }

  cout << "interation number = " << count1 << "\n";

/* unflag pixels of small counts */

  for(count1 = 0; count1 < naxis[1] * naxis[2]; count1++) {
    if(cr_flag[count1] == 0) continue;
    if(obs_data_init[count1] - obs_data[count1] < cr_count) cr_flag[count1] = 0;
  }

/* output image file */

  strcpy(cr_list, argv[3]);
  remove(cr_list);
  ofstream fout(cr_list);
  if(!fout) {
    cout << "Cannot open file " << cr_list << ".\n";
    return 2;
  }

  fout << "# x_start x_end y\n";

  for(count2 = 0; count2 < naxis[2]; count2++) {
    for(count1 = 0; count1 < naxis[1]; count1++) {
      if(cr_flag[count1 + count2 * naxis[1]] == 0) continue;

      fout << count1 + 1 << " ";
      while(cr_flag[count1 + 1 + count2 * naxis[1]] == 1 && count1 + 1 < naxis[1]) count1++;
      fout << count1 + 1 << " " << count2 + 1 << "\n";
    }
  }

  fout.close();

  return 0;
}
