/*
program for detecting cosmic rays (difference ver.)

detection argorithm: 
  1. subtract frame1 by frame2 (using corrected frames if 2nd or later)
  2. detect cosmic rays with the following process in x and y direction: detect a pixel whose value is larger than sigma_threshold of the previous and next 3 pixels
  3. correct frames internally
  4. iterate process 1-3
*/

#include "FiberDetectCosmicDiff.h"

void input_v_tmp_x(vector<double> &v_input, vector<double> &v_output, int x, int y, int naxis1)
{
  v_output[0] = v_input[x - 3 + y * naxis1];
  v_output[1] = v_input[x - 2 + y * naxis1];
  v_output[2] = v_input[x - 1 + y * naxis1];
  v_output[3] = v_input[x + 1 + y * naxis1];
  v_output[4] = v_input[x + 2 + y * naxis1];
  v_output[5] = v_input[x + 3 + y * naxis1];
}

void input_v_tmp_x2(vector<double> &v_input, vector<double> &v_output, int x, int y, int naxis1)
{
  v_output[0] = v_input[x - 3 + y * naxis1];
  v_output[1] = v_input[x - 2 + y * naxis1];
  v_output[2] = v_input[x - 1 + y * naxis1];
  v_output[3] = v_input[x + 2 + y * naxis1];
  v_output[4] = v_input[x + 3 + y * naxis1];
  v_output[5] = v_input[x + 4 + y * naxis1];
}

void input_v_tmp_y(vector<double> &v_input, vector<double> &v_output, int x, int y, int naxis1)
{
  v_output[0] = v_input[x + (y - 3) * naxis1];
  v_output[1] = v_input[x + (y - 2) * naxis1];
  v_output[2] = v_input[x + (y - 1) * naxis1];
  v_output[3] = v_input[x + (y + 1) * naxis1];
  v_output[4] = v_input[x + (y + 2) * naxis1];
  v_output[5] = v_input[x + (y + 3) * naxis1];
}

void input_v_tmp_y2(vector<double> &v_input, vector<double> &v_output, int x, int y, int naxis1)
{
  v_output[0] = v_input[x + (y - 3) * naxis1];
  v_output[1] = v_input[x + (y - 2) * naxis1];
  v_output[2] = v_input[x + (y - 1) * naxis1];
  v_output[3] = v_input[x + (y + 2) * naxis1];
  v_output[4] = v_input[x + (y + 3) * naxis1];
  v_output[5] = v_input[x + (y + 4) * naxis1];
}

void remove_cosmicrays(vector<double> &v_data_in, vector<double> &v_data_out, vector<int> &v_flag, int naxis[], int check_buff)
{
  int count1, count2, count3, pos_xy, cr_length;

  v_data_out = v_data_in;

  for(count2 = 0; count2 < naxis[2]; count2++) {
    for(count3 = check_buff; count3 < naxis[1] - check_buff; count3++) {
      pos_xy = count3 + count2 * naxis[1];
      if(v_flag[pos_xy] == 0) continue;

      cr_length = 1;
      while(v_flag[pos_xy + cr_length] == 1) {
	cr_length++;
      }

      for(count1 = count3; count1 < count3 + cr_length; count1++) {
	v_data_out[count1 + count2 * naxis[1]] = func_linear_approx((double)count3 - 1, (double)count3 + cr_length, v_data_in[count3 - 1 + count2 * naxis[1]], v_data_in[count3 + cr_length + count2 * naxis[1]], (double)count1);
      }

      count3 += cr_length - 1;
    }
  }
}

int main(int argc, char *argv[])
{
  if (argc != 8) {
    cout << "The number of argments does not match.\n";
    cout << "Usage: FiberDetectCosmicDiff input1_fits input2_fits output1_dat output2_dat cr_thresh_count cr_thresh_sigma cr_thresh_sigma2\n";
    return 1;
  }

  int count1, count2, count3, naxis[3], n_iterate = 100, pos_xy, tmp_int;
  double *get_data1, *get_data2, exp_time[3], cr_count = atof(argv[5]), cr_sigma = atof(argv[6]), cr_sigma2 = atof(argv[7]), tmp_double;
  char obs_frame1[char_length], obs_frame2[char_length], cr_list1[char_length], cr_list2[char_length], get_char[char_length] = "", tmp_char[char_length] = "", *p_char;
  string detect_flag;
  vector<int> cr_flag1, cr_flag2;
  vector<double> obs_data1, obs_data1_init, obs_data2, obs_data2_init, data_subt, v_tmp(6, 0);

/* open input fits files */

  strcpy(obs_frame1, argv[1]);
  strcpy(obs_frame2, argv[2]);
  get_naxis(obs_frame1, naxis);

  get_data1 = new double [naxis[1] * naxis[2]];
  if (!get_data1) {
    cout << "Memory error.\n";
    return 2;
  }

  get_data2 = new double [naxis[1] * naxis[2]];
  if (!get_data2) {
    cout << "Memory error.\n";
    return 2;
  }

  tmp_int = read_header(obs_frame1, "EXPTIME ", get_char);
  if(tmp_int) {
    cout << "fits file read error.\n";
    return 2;
  }

  p_char = get_char;
  strncpy(tmp_char, p_char + 11, 21);
  exp_time[1] = atof(tmp_char);
//  cout << "obs_frame = " << obs_frame << " exp_time = " << exp_time << "\n";

  tmp_int = read_header(obs_frame2, "EXPTIME ", get_char);
  if(tmp_int) {
    cout << "fits file read error.\n";
    return 2;
  }

  p_char = get_char;
  strncpy(tmp_char, p_char + 11, 21);
  exp_time[2] = atof(tmp_char);
//  cout << "obs_frame = " << obs_frame << " exp_time = " << exp_time << "\n";
  tmp_double = exp_time[1] / exp_time[2];

  obs_data1_init.resize(naxis[1] * naxis[2], 0);
  obs_data2_init.resize(naxis[1] * naxis[2], 0);
  data_subt.resize(naxis[1] * naxis[2], 0);

  open_fitsfile(obs_frame1, naxis[1], naxis[2], get_data1);
  open_fitsfile(obs_frame2, naxis[1], naxis[2], get_data2);
  for (count1 = 0; count1 < naxis[1] * naxis[2]; count1++) {
    obs_data1_init[count1] = get_data1[count1];
    obs_data2_init[count1] = get_data2[count1] * tmp_double;
  }

  delete [] get_data1;
  delete [] get_data2;

/* prepare for main loop */

  obs_data1 = obs_data1_init;
  obs_data2 = obs_data2_init;
  cr_flag1.resize(naxis[1] * naxis[2], 0);
  cr_flag2.resize(naxis[1] * naxis[2], 0);

/* main roop */

  for(count1 = 0; count1 < n_iterate; count1++) {
    detect_flag = "false";
// subtract frame1 by frame2
    for(count2 = 0; count2 < naxis[1] * naxis[2]; count2++) {
      data_subt[count2] = obs_data1[count2] - obs_data2[count2];
    }

// check 1 pixel
// check in x direction
    for(count2 = 0; count2 < naxis[2]; count2++) {
      for(count3 = check_buff; count3 < naxis[1] - check_buff; count3++) {
	pos_xy = count3 + count2 * naxis[1];

	input_v_tmp_x(data_subt, v_tmp, count3, count2, naxis[1]);
	if(data_subt[pos_xy] - func_mean(v_tmp) > func_stddev(v_tmp) * cr_sigma) {
	  if(cr_flag1[pos_xy] == 0) detect_flag = "true";
	  cr_flag1[pos_xy] = 1;
	}
	if(data_subt[pos_xy] - func_mean(v_tmp) < -func_stddev(v_tmp) * cr_sigma) {
	  if(cr_flag2[pos_xy] == 0) detect_flag = "true";
	  cr_flag2[pos_xy] = 1;
	}
      }
    }

    remove_cosmicrays(obs_data1_init, obs_data1, cr_flag1, naxis, check_buff);
    remove_cosmicrays(obs_data2_init, obs_data2, cr_flag2, naxis, check_buff);

// subtract frame1 by frame2
    for(count2 = 0; count2 < naxis[1] * naxis[2]; count2++) {
      data_subt[count2] = obs_data1[count2] - obs_data2[count2];
    }

// check in y direction
    for(count3 = 0; count3 < naxis[1]; count3++) {
      for(count2 = check_buff; count2 < naxis[2] - check_buff; count2++) {
	pos_xy = count3 + count2 * naxis[1];

	input_v_tmp_y(data_subt, v_tmp, count3, count2, naxis[1]);
	if(data_subt[pos_xy] - func_mean(v_tmp) > func_stddev(v_tmp) * cr_sigma) {
	  if(cr_flag1[pos_xy] == 0) detect_flag = "true";
	  cr_flag1[pos_xy] = 1;
	}
	if(data_subt[pos_xy] - func_mean(v_tmp) < -func_stddev(v_tmp) * cr_sigma) {
	  if(cr_flag2[pos_xy] == 0) detect_flag = "true";
	  cr_flag2[pos_xy] = 1;
	}
      }
    }

    remove_cosmicrays(obs_data1_init, obs_data1, cr_flag1, naxis, check_buff);
    remove_cosmicrays(obs_data2_init, obs_data2, cr_flag2, naxis, check_buff);

    if(detect_flag == "true") continue;

// check 2 pixels
// check in x direction
    for(count2 = 0; count2 < naxis[2] - 1; count2++) {
      for(count3 = check_buff; count3 < naxis[1] - (check_buff + 1); count3++) {
	pos_xy = count3 + count2 * naxis[1];

	input_v_tmp_x2(data_subt, v_tmp, count3, count2, naxis[1]);
	if(data_subt[pos_xy] - func_mean(v_tmp) > func_stddev(v_tmp) * cr_sigma2) {
	  if(cr_flag1[pos_xy] == 0 || cr_flag1[pos_xy + 1] == 0) detect_flag = "true";
	  cr_flag1[pos_xy] = 1;
	  cr_flag1[pos_xy + 1] = 1;
	}
	if(data_subt[pos_xy] - func_mean(v_tmp) < -func_stddev(v_tmp) * cr_sigma2) {
	  if(cr_flag2[pos_xy] == 0 || cr_flag2[pos_xy + 1] == 0) detect_flag = "true";
	  cr_flag2[pos_xy] = 1;
	  cr_flag2[pos_xy + 1] = 1;
	}
      }
    }

    remove_cosmicrays(obs_data1_init, obs_data1, cr_flag1, naxis, check_buff);
    remove_cosmicrays(obs_data2_init, obs_data2, cr_flag2, naxis, check_buff);

// subtract frame1 by frame2
    for(count2 = 0; count2 < naxis[1] * naxis[2]; count2++) {
      data_subt[count2] = obs_data1[count2] - obs_data2[count2];
    }

// check in y direction
    for(count3 = 0; count3 < naxis[1]; count3++) {
      for(count2 = check_buff; count2 < naxis[2] - (check_buff + 1); count2++) {
	pos_xy = count3 + count2 * naxis[1];

	input_v_tmp_y2(data_subt, v_tmp, count3, count2, naxis[1]);
	if(data_subt[pos_xy] - func_mean(v_tmp) > func_stddev(v_tmp) * cr_sigma2) {
	  if(cr_flag1[pos_xy] == 0 || cr_flag1[pos_xy + 1] == 0) detect_flag = "true";
	  cr_flag1[pos_xy] = 1;
	  cr_flag1[pos_xy + 1] = 1;
	}
	if(data_subt[pos_xy] - func_mean(v_tmp) < -func_stddev(v_tmp) * cr_sigma2) {
	  if(cr_flag2[pos_xy] == 0 || cr_flag2[pos_xy + 1] == 0) detect_flag = "true";
	  cr_flag2[pos_xy] = 1;
	  cr_flag2[pos_xy + 1] = 1;
	}
      }
    }

    remove_cosmicrays(obs_data1_init, obs_data1, cr_flag1, naxis, check_buff);
    remove_cosmicrays(obs_data2_init, obs_data2, cr_flag2, naxis, check_buff);

    if(detect_flag == "false") break;
  }

  cout << "interation number = " << count1 << "\n";

/* unflag pixels of small counts */

  for(count1 = 0; count1 < naxis[1] * naxis[2]; count1++) {
    if(cr_flag1[count1] == 0 || cr_flag2[count1] == 0) continue;
    if(obs_data1_init[count1] - obs_data1[count1] < cr_count) cr_flag1[count1] = 0;
    if(obs_data2_init[count1] - obs_data2[count1] < cr_count) cr_flag2[count1] = 0;
  }

/* output cosmicray position list */

// frame1
  strcpy(cr_list1, argv[3]);
  remove(cr_list1);
  ofstream fout1(cr_list1);
  if(!fout1) {
    cout << "Cannot open file " << cr_list1 << ".\n";
    return 2;
  }

  fout1 << "# x_start x_end y\n";

  for(count2 = 0; count2 < naxis[2]; count2++) {
    for(count1 = 0; count1 < naxis[1]; count1++) {
      if(cr_flag1[count1 + count2 * naxis[1]] == 0) continue;

      fout1 << count1 + 1 << " ";
      while(cr_flag1[count1 + 1 + count2 * naxis[1]] == 1 && count1 + 1 < naxis[1]) count1++;
      fout1 << count1 + 1 << " " << count2 + 1 << "\n";
    }
  }

  fout1.close();

//frame2
  strcpy(cr_list2, argv[4]);
  remove(cr_list2);
  ofstream fout2(cr_list2);
  if(!fout2) {
    cout << "Cannot open file " << cr_list2 << ".\n";
    return 2;
  }

  fout2 << "# x_start x_end y\n";

  for(count2 = 0; count2 < naxis[2]; count2++) {
    for(count1 = 0; count1 < naxis[1]; count1++) {
      if(cr_flag2[count1 + count2 * naxis[1]] == 0) continue;

      fout2 << count1 + 1 << " ";
      while(cr_flag2[count1 + 1 + count2 * naxis[1]] == 1 && count1 + 1 < naxis[1]) count1++;
      fout2 << count1 + 1 << " " << count2 + 1 << "\n";
    }
  }

  fout2.close();

  return 0;
}
