#include "FiberGaussImage.h"

int main(int argc, char *argv[])
{
  if (argc != 5 && argc != 6) {
    cout << "The number of argments does not match.\n";
    cout << "  Usage: GaussImage output_fits offset_x offset_y FWHM noise.\n";
    cout << "                                center=0 center=0 (in fiber unit)\n";
    return 1;
  }

  int count1, count2, count3, count4, pos_x, pos_y;
  double xc, yc, dx, dy, total = 0, sigma, sigma2double, noise = 0;
  double offset_x = atof(argv[2]) * 2, offset_y = atof(argv[3]) * 2, FWHM = atof(argv[4]) * 2;
  double a1, a2, b, g1;
  double *out_data;
  char out_frame[char_length];
  
/* initialize */

  strcpy(out_frame, argv[1]);

  out_data = new double [naxis1_out * naxis2_out];
  if (!out_data) {
    cout << "Memory error.\n";
    return 2;
  }

  sigma = FWHM / sqrt(8 * log(2));
  sigma2double = sigma * sigma * 2;
  dx = dy = 2 * core_clad_ratio / total_step; // clad radius = 1

  for(count1 = 0; count1 < naxis1_out * naxis2_out; count1++) out_data[count1] = 0;

  if (argc == 6) noise = atof(argv[5]);
  srand((unsigned int)time(NULL));

/* calculate fiber flux  */

  for(count2 = 1; count2 <= 7; count2++) {
    for(count1 = 1; count1 < 7 + count2; count1++) {
      pos_x = count1 * 2 - count2 - 7;
      pos_y = 14 - count2 * 2;

      for(count4 = 0; count4 < total_step; count4++) {
        yc = ((double)count4 - (double)total_step * 0.5 + 0.5) * dy;
        for(count3 = 0; count3 < total_step; count3++) {
          xc = ((double)count3 - (double)total_step * 0.5 + 0.5) * dx;

          if(func_square(xc) + func_square(yc) <= func_square(core_clad_ratio)) {
	    total += exp(-((func_square(xc + (double)pos_x - offset_x) + func_square(yc + ((double)pos_y - offset_y) * sqrt(3) * 0.5)) / sigma2double));
	  }
        }
      }

  /* noise */

    do {
      a1 = 2.0 * rand() / RAND_MAX - 1.0;
      a2 = 2.0 * rand() / RAND_MAX - 1.0;
      b = a1 * a1 + a2 * a2;
    } while(b >= 1.0);
    b = sqrt( (-2.0 * log( b ) ) / b );
    g1 = a1 * b;

    out_data[(pos_x + 12) + (pos_y + 12) * naxis2_out] = total * dx * dy / (pi * sigma2double) + noise * g1;
    total = 0;
    }
  }

  for(count2 = 1; count2 <= 6; count2++) {
    for(count1 = 1; count1 < 14 - count2; count1++) {
      pos_x = count1 * 2 + count2 - 14;
      pos_y = - count2 * 2;

      for(count4 = 0; count4 < total_step; count4++) {
        yc = ((double)count4 - (double)total_step * 0.5 + 0.5) * dy;
        for(count3 = 0; count3 < total_step; count3++) {
          xc = ((double)count3 - (double)total_step * 0.5 + 0.5) * dx;

          if(func_square(xc) + func_square(yc) <= func_square(core_clad_ratio)) {
	    total += exp(-((func_square(xc + (double)pos_x - offset_x) + func_square(yc + ((double)pos_y - offset_y) * sqrt(3) * 0.5)) / sigma2double));
	  }
        }
      }

  /* noise */

    do {
      a1 = 2.0 * rand() / RAND_MAX - 1.0;
      a2 = 2.0 * rand() / RAND_MAX - 1.0;
      b = a1 * a1 + a2 * a2;
    } while(b >= 1.0);
    b = sqrt( (-2.0 * log( b ) ) / b );
    g1 = a1 * b;

    out_data[(pos_x + 12) + (pos_y + 12) * naxis2_out] = total * dx * dy / (pi * sigma2double) + noise * g1;
    total = 0;
    }
  }

/* output image file */

  remove(out_frame);
  create_fitsfile(out_frame, naxis1_out, naxis2_out, out_data);

  delete [] out_data;

  return 0;
}
