#include "FiberImageReal.h"

void calc_RGB(double value, double range_min, double range_max, int linear_log, char *RGB_code)
{
  int color;
  double LogExponent = 1000, value_frac;
  char output[char_length];

  value_frac = (value - range_min) / (range_max - range_min);

  if (linear_log == 1) color = (int)(value_frac * 1024);
  else if (linear_log == 2) color = (int)((log(LogExponent * value_frac + 1)) / log(LogExponent) * 1024);
  else color = 0;

  if (color < 0) color = 0;
  else if (color > 1023) color = 1023;

  if (color >= 256 * 0 && color < 256 * 1) sprintf(output, "0000%02X", color);
  else if (color >= 256 * 1 && color < 256 * 2) sprintf(output, "%02X00%02X", color - 256, 511 - color);
  else if (color >= 256 * 2 && color < 256 * 3) sprintf(output, "FF%02X00", color - 512);
  else sprintf(output, "FFFF%02X", color - 768);

  strcpy(RGB_code, output);
}

int main(int argc, char *argv[])
{
  if (argc != 6) {
    cout << "The number of argments does not match.\n";
    cout << "  Usage: FiberImageReal input_fits output_name range_min range_max [linear or log].\n";
    return 1;
  }

/* initialize */

  int count1, count2, linear_log = 0, fiber_x, fiber_y;
  double center_x, center_y, range_min = atof(argv[3]), range_max = atof(argv[4]), *get_data;
  vector<double> obs_data;
  char obs_frame[char_length], char_tmp[char_length], RGB_code[char_length], out_system[char_length * 32];
  
  get_data = new double [naxis1_in * naxis2_in];
  if (!get_data) {
    cout << "Memory error.\n";
    return 2;
  }

  if (!strcmp(argv[5], "linear")) linear_log = 1;
  else if (!strcmp(argv[5], "log")) linear_log = 2;
  else {
    cout << "The last argment must be linear or log.\n";
    cout << "  Usage: FiberImageReal input_fits output_name range_min range_max [linear or log].\n";
    return 1;
  }

/* open input fits file */

  strcpy (obs_frame, argv[1]);

  open_fitsfile(obs_frame, naxis1_in, naxis2_in, get_data);
  for (count1 = 0; count1 < naxis1_in * naxis2_in; count1++) {
    obs_data.push_back(get_data[count1]);
  }

/* preamble */

  strcpy (out_system, "convert -size 1625x1625 xc:white -stroke black -strokewidth 10 ");

/* draw circles */

  for(count2 = 1; count2 <= 7; count2++) {
    for(count1 = 1; count1 < 7 + count2; count1++) {
      fiber_x = count1 * 2 - count2 + 6;
      fiber_y = count2 * 2 - 1;
      center_x = fiber_x * r_fiber;
      center_y = fiber_y * r_fiber * sqrt3_div2;
      calc_RGB(get_data[fiber_x - 1 + naxis1_in * (fiber_y - 1)], range_min, range_max, linear_log, RGB_code);

      sprintf (char_tmp, "-fill '#%s' -draw 'circle %.2f,%.2f %2.f,%.2f' ", RGB_code, center_x, image_max_y - center_y + 1, center_x + r_core, image_max_y - center_y + 1);
      strcat (out_system, char_tmp);
      
      memset(char_tmp, '\0', char_length * sizeof(char));
    }
  }

  for(count2 = 1; count2 <= 6; count2++) {
    for(count1 = 1; count1 < 14 - count2; count1++) {
      fiber_x = count1 * 2 + count2 - 1;
      fiber_y = count2 * 2 + 13;
      center_x = fiber_x * r_fiber;
      center_y = fiber_y * r_fiber * sqrt3_div2;
      calc_RGB(get_data[fiber_x - 1 + naxis1_in * (fiber_y - 1)], range_min, range_max, linear_log, RGB_code);

      sprintf (char_tmp, "-fill '#%s' -draw 'circle %.2f,%.2f %2.f,%.2f' ", RGB_code, center_x, image_max_y - center_y + 1, center_x + r_core, image_max_y - center_y + 1);
      strcat (out_system, char_tmp);
      
      memset(char_tmp, '\0', char_length * sizeof(char));
    }
  }

/* colorbar */

  strcat (out_system, "-strokewidth 1 ");

  for(count1 = 1; count1 < image_max_x; count1++) {
    calc_RGB(count1, 0, image_max_x, 1, RGB_code);

    sprintf (char_tmp, "-stroke '#%s' -draw 'line %d,1550 %d,1620' ", RGB_code, count1, count1);
    strcat (out_system, char_tmp);
      
    memset(char_tmp, '\0', char_length * sizeof(char));
  }

  strcat (out_system, "-stroke black -strokewidth 7 -draw 'stroke-linecap square line 5,1550 5,1620 line 5,1620 1620,1620 line 1620,1620 1620,1550 line 1620,1550 5,1550' ");

/* output */

  strcat (out_system, argv[2]);
  remove (argv[2]);

//cout << out_system << "\n";

  system (out_system);

  delete [] get_data;

  return 0;
}

