#include "FiberRejectCosmic.h"

int main(int argc, char *argv[])
{
  if (argc == 3) {
    cout << "The number of argments does not match.\n";
    cout << "Usage: FiberRejectCosmic input_fits output_name cr_list\n";
    return 1;
  }

  int count1, naxis[3], cr_x_start, cr_x_end, cr_y;
  double *get_data, *out_data, *out_diff;
  vector<double> obs_data, obs_data_init;
  char obs_frame[char_length], out_frame[char_length], out_diff_frame[char_length] = "diff-", tmp_char[char_length];
  string str_tmp, str_comb = "";
  string::size_type comment_start = 0;

/* open input fits file */

  strcpy (obs_frame, argv[1]);
  get_naxis(obs_frame, naxis);

  get_data = new double [naxis[1] * naxis[2]];
  if (!get_data) {
    cout << "Memory error.\n";
    return 2;
  }

  open_fitsfile(obs_frame, naxis[1], naxis[2], get_data);
  for (count1 = 0; count1 < naxis[1] * naxis[2]; count1++) {
    obs_data.push_back(get_data[count1]);
    obs_data_init.push_back(get_data[count1]);
  }

  delete [] get_data;

/* reject cosmic rays */

  ifstream fin(argv[3]);
  if(!fin) {
    cout << "Cannot open " << argv[3] << ".\n";
    return 1;
  }

  while (getline(fin, str_tmp)) {
    if ((comment_start = str_tmp.find('#')) != string::size_type(-1)) str_tmp = str_tmp.substr(0, comment_start);
    if (str_tmp.empty()) continue;

    strcpy (tmp_char, str_tmp.c_str());
    sscanf (tmp_char, "%d %d %d", &cr_x_start, &cr_x_end, &cr_y);

    for (count1 = cr_x_start; count1 <= cr_x_end; count1++) {
      obs_data[count1 - 1 + (cr_y - 1) * naxis[1]] = func_linear_approx((double)cr_x_start - 2, (double)cr_x_end, obs_data[cr_x_start - 2 + (cr_y - 1) * naxis[1]], obs_data[cr_x_end + (cr_y - 1) * naxis[1]], (double)count1 - 1);
    }
  }

  fin.close();

/* output image file */

  strcpy(out_frame, argv[2]);
  strcat(out_diff_frame, out_frame);

  out_data = new double [naxis[1] * naxis[2]];
  if (!out_data) {
    cout << "Memory error.\n";
    return 2;
  }

  out_diff = new double [naxis[1] * naxis[2]];
  if (!out_diff) {
    cout << "Memory error.\n";
    return 2;
  }

  for(count1 = 0; count1 < naxis[1] * naxis[2]; count1++) {
    out_data[count1] = obs_data[count1];
    out_diff[count1] = obs_data_init[count1] - obs_data[count1];
  }

  remove(out_frame);
  remove(out_diff_frame);
  create_fitsfile(obs_frame, out_frame, naxis[1], naxis[2], out_data);
  create_fitsfile(obs_frame, out_diff_frame, naxis[1], naxis[2], out_diff);

  delete [] out_data;
  delete [] out_diff;

  return 0;
}
