#include "FiberSkySubtract.h"

int main(int argc, char *argv[])
{
  if(argc != 5) {
    cout << "The number of argments does not match.\n";
    cout << "  Usage: FiberSkySubtract input_fits output_fits column_sky mask_fits.\n";
    cout << "    The column_sky's separator is \",\". (example: 34,56,100)\n";
    return 1;
  }

  int count1, count2, naxis[3], tmp_int, smooth_factor = 30;
  double *get_data, *get_mask, *out_data;
  vector<int> sky_fiber;
  vector<double> sky_flux, sky_flux_smooth, tmp_sky;
  vector<double>::iterator p;
  char input_frame[char_length], input_mask[char_length], output_frame[char_length], sky_frame[char_length] = "sky-", *token;

/* input fits file */  

  strcpy(input_frame, argv[1]);
  strcpy(input_mask, argv[4]);
  strcpy(output_frame, argv[2]);

  get_naxis(input_frame, naxis);

  get_data = new double [naxis[1] * naxis[2]];
  if(!get_data) {
    cout << "Memory error.\n";
    return 2;
  }

  get_mask = new double [naxis[1] * naxis[2]];
  if(!get_mask) {
    cout << "Memory error.\n";
    return 2;
  }

  out_data = new double [naxis[1] * naxis[2]];
  if(!out_data) {
    cout << "Memory error.\n";
    return 2;
  }

  open_fitsfile(input_frame, naxis[1], naxis[2], get_data);
  open_fitsfile(input_mask, naxis[1], naxis[2], get_mask);

/* calculate sky spectrum */

  token = strtok(argv[3], ",");
  tmp_int = atoi(token);
  if(tmp_int > 0 && tmp_int <= num_fiber) sky_fiber.push_back(tmp_int);
  while(token != NULL) {
    token = strtok(NULL, ",");
    if(token != NULL) {
      tmp_int = atoi(token);
      if(tmp_int > 0 && tmp_int <= num_fiber) sky_fiber.push_back(tmp_int);
    }
  }

/*
  for(count1 = 0; count1 < (int)sky_fiber.size(); count1++) cout << sky_fiber[count1] << " ";
  cout << "\n";
*/

  if((int)sky_fiber.size() == 0) {
    for(count1 = 0; count1 < naxis[1]; count1++) {
      sky_flux.push_back(0);
    }
  }
  else{
    for(count1 = 0; count1 < naxis[1]; count1++) {
      for(count2 = 0; count2 < (int)sky_fiber.size(); count2++) {
	if(get_mask[count1 + (sky_fiber[count2] - 1) * naxis[1]] == 0) tmp_sky.push_back(get_data[count1 + (sky_fiber[count2] - 1) * naxis[1]]);
      }

//cout << "x = " << count1 << " tmp_sky.size = " << tmp_sky.size() << "\n";
      if(tmp_sky.size() == 0) sky_flux.push_back(0);
      else sky_flux.push_back(func_median(tmp_sky));

      p = tmp_sky.begin();
      tmp_sky.erase(p, p + tmp_sky.size());
    }
  }
    
/* smooth sky spectrum */

  sky_flux_smooth.push_back((smooth_factor * sky_flux[0] + sky_flux[1]) / (smooth_factor + 1));
  for(count1 = 1; count1 < naxis[1] - 1; count1++) {
    sky_flux_smooth.push_back((sky_flux[count1 - 1] + smooth_factor * sky_flux[count1] + sky_flux[count1 + 1]) / (smooth_factor + 2));
  }
  sky_flux_smooth.push_back((sky_flux[naxis[1] - 2] + smooth_factor * sky_flux[naxis[1] - 1]) / (smooth_factor + 2));

/* subtract sky emission */

  for(count1 = 0; count1 < naxis[1]; count1++) {
    for(count2 = 0; count2 < naxis[2]; count2++) {
//      out_data[count1 + naxis[1] * count2] = get_data[count1 + naxis[1] * count2] - sky_flux[count1];
      out_data[count1 + naxis[1] * count2] = get_data[count1 + naxis[1] * count2] - sky_flux_smooth[count1];
    }
  }

/* ouotput file */

  strcat(sky_frame, output_frame);
  token = strstr(sky_frame, "-sky.fits");
  if(token == NULL) strcat(sky_frame, ".dat");
  else {
    *token = '\0';
    strcat(sky_frame, ".dat");
  }

//  cout << "sky_frame = " << sky_frame << "\n";
  remove(sky_frame);
  ofstream fout(sky_frame);
  if(!fout) {
    cout << "Cannot open file " << sky_frame << ".\n";
    return 2;
  }

  for(count1 = 0; count1 < naxis[1]; count1++) fout << sky_flux_smooth[count1] << "\n";

  fout.close();

  remove(output_frame);
  create_fitsfile(input_frame, output_frame, naxis[1], naxis[2], out_data);

  delete [] get_data;
  delete [] get_mask;
  delete [] out_data;

  return 0;
}
