#include "FiberSpotToImage.h"

int main(int argc, char *argv[])
{
  if (argc != 7) {
    cout << "The number of argments does not match.\n";
    cout << "  Usage: FiberSpotToImage input_fits init_x init_y center_range bias_value binning.\n";
    return 1;
  }

  int count1, count2, init_tmp_x = 0, init_tmp_y = 0, tmp = 0;
  int center_range = atoi(argv[4]), binning = atoi(argv[6]), fiber_pos_x[num_fiber], fiber_pos_y[num_fiber];
  double *get_data, *out_data, center_x = atof(argv[2]) - 1, center_y = atof(argv[3]) - 1, bias = atof(argv[5]), previous_y = -10;
  vector<double> obs_data, fiber_flux;
  char obs_frame[char_length], out_frame[char_length], pos_dat[char_length], *dot_position;
  
  if (binning != 1 && binning != 2) {
    cout << "binning = 1 or 2\n";
    return 1;
  }

  get_data = new double [naxis1_in[binning] * naxis2_in[binning]];
  if (!get_data) {
    cout << "Memory error.\n";
    return 2;
  }

  out_data = new double [naxis1_out * naxis2_out];
  if (!out_data) {
    cout << "Memory error.\n";
    return 2;
  }

/* open input fits file */

  strcpy (obs_frame, argv[1]);

  open_fitsfile(obs_frame, naxis1_in[binning], naxis2_in[binning], get_data);
  for (count1 = 0; count1 < naxis1_in[binning] * naxis2_in[binning]; count1++) {
    obs_data.push_back(get_data[count1] - bias);
    }

/* set output file name */

  dot_position = strrchr(obs_frame, '.');

  strncpy(out_frame, obs_frame, strlen(obs_frame) - strlen(dot_position));
  strcat(out_frame, "-image.fits");
  remove(out_frame);

  strncpy(pos_dat, obs_frame, strlen(obs_frame) - strlen(dot_position));
  strcat(pos_dat, "-pos.dat");
  remove(pos_dat);

  ofstream fout1(pos_dat);
  if(!fout1) {
    cout << "Cannot open " << pos_dat << ".\n";
    return 1;
  }

  fout1 << naxis1_in[binning] << " " << naxis2_in[binning] << "\n";

/* calculate gravity cneter */

  for(count1 = 0; count1 < 127; count1++) {

    do {
      init_tmp_x = (int)center_x;
      init_tmp_y = (int)center_y;

      center_x = func_center(obs_data, naxis1_in[binning], naxis2_in[binning], init_tmp_x, init_tmp_y, center_range, 1);
      center_y = func_center(obs_data, naxis1_in[binning], naxis2_in[binning], init_tmp_x, init_tmp_y, center_range, 2);

//  cout << "center_x = " << center_x + 1 << " center_y = " << center_y + 1 << "\n";

    } while ((abs(center_x - init_tmp_x) > 1 || abs(center_y - init_tmp_y) > 1) && center_y - previous_y > 4 / binning);

    fout1 << center_x << " " << center_y << "\n";
    previous_y = center_y;

/* calculate fiber flux */

    fiber_flux.push_back(func_box_flux(obs_data, naxis1_in[binning], naxis2_in[binning], center_x, center_y, center_range));
    if (center_y + 1 >= naxis2_in[binning]) fiber_flux[count1] = 0;

//    cout << "count1 = " << count1 << " fiber_flux = " << fiber_flux[count1] << "\n";

    center_y += step_y[binning];
    if(count1 == 41) {
      center_x += 1 / binning;
      center_y += 86 / binning;
    }
    if(count1 == 84) {
      center_x += 1 / binning;
      center_y += 86 / binning;
    }
  }

/* fiber position library */

  for(count2 = 1; count2 <= 7; count2++) {
    for(count1 = 1; count1 < 7 + count2; count1++) {
      fiber_pos_x[count1 + tmp] = count1 * 2 + 6 - count2;
      fiber_pos_y[count1 + tmp] = 27 - count2 * 2;
    }
    tmp += count1 - 1;
//    cout << "tmp = " << tmp << "\n";
  }

  for(count2 = 1; count2 <= 6; count2++) {
    for(count1 = 1; count1 < 14 - count2; count1++) {
      fiber_pos_x[count1 + tmp] = count1 * 2 + count2 - 1;
      fiber_pos_y[count1 + tmp] = 13 - count2 * 2;
    }
    tmp += count1 - 1;
//    cout << "tmp = " << tmp << "\n";
  }

/* align in fiberbundle format */

  for(count1 = 0; count1 < naxis1_out * naxis2_out; count1++) out_data[count1] = 0;

  for(count1 = 0; count1 < num_fiber; count1++) {
    out_data[fiber_pos_x[fiber_id[count1]] - 1 + naxis2_out * (fiber_pos_y[fiber_id[count1]] - 1)] = fiber_flux[count1];
  }

/* output image file */

  fout1.close();

  create_fitsfile(obs_frame, out_frame, naxis1_out, naxis2_out, out_data);

  delete [] get_data;
  delete [] out_data;

  return 0;
}
